//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringBufferInputStream;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.oooneida.wb.dom.DocumentSource;
import org.oooneida.wb.dom.events.MutationListener;
import org.w3c.dom.DOMException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.bootstrap.DOMImplementationRegistry;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSInput;
import org.w3c.dom.ls.LSOutput;
import org.w3c.dom.ls.LSParser;
import org.w3c.dom.ls.LSSerializer;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * A singleton collection of named XML Documents.
 * 
 * @author JHC
 * @version 20051111/JHC - Added MutationListener in scrub().
 * @version 20050708/JHC
 */
public class Library implements EntityResolver {
	private Library() {
		super();
	}

	private static final String schemaFile = "MyLibraryElement.xsd";

	private static Library library = new Library();

	private static Hashtable documents = new Hashtable(32);

	private static DocumentSource documentSource;

	private static DocumentBuilder documentBuilder;

	/**
	 * Sets the DocumentSource for finding an InputSource for a given document
	 * name.
	 */
	public void setSource(DocumentSource src) {
		documentSource = src;
	}

	/**
	 * Returns a Document with the given name, with its root node and children
	 * stripped down to Elements only, or <TT>null</TT> if a Document with the
	 * given name cannot be found.
	 * 
	 * @param docname
	 *            The document name, which may have the format <TT>
	 *            identifier.suffix</TT> to simplify searching for a file to
	 *            load (the suffix will be discarded for searching the library
	 *            of already loaded Documents).
	 */
	public static org.w3c.dom.Document getDocument(String docname) {
		int n = docname.lastIndexOf('.');
		String dname = (n < 0) ? docname : docname.substring(0, n);
		Object obj = documents.get(dname);
		if (obj instanceof Document)
			return (Document) obj;
		if (documentSource == null) {
			System.err.println("No DocumentSource defined");
			return null;
		}
		return load(documentSource.getSource(docname));
	}

	/**
	 * Recursively remove all non-Element nodes of the tree below the given
	 * Element and add a MutationListener (if any) to the Element.
	 */
	private static void scrub(org.w3c.dom.Element el) {
		MutationListener.forElement(el);
		NodeList nl = el.getChildNodes();
		for (int i = nl.getLength() - 1; i >= 0; i--) {
			Node n = nl.item(i);
			if (n instanceof Element) {
				Element el2 = (Element) n;
				scrub(el2);
			} else
				el.removeChild(n);
		}
	}


	/** Returns the DocumentBuilder with lazy initialization. */
	public static DocumentBuilder getDocumentBuilder() {
		if (documentBuilder != null)
			return documentBuilder;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(true);

		SchemaFactory schemaFactory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");
		Schema schema = null;
		try {
			schema = schemaFactory.newSchema(new StreamSource(
					Library.class.getResourceAsStream(schemaFile)));
			factory.setSchema(schema);
		} catch (SAXException e1) {
			e1.printStackTrace();
		}

		try {
			documentBuilder = factory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			System.err.println(e);
			return null;
		}
		documentBuilder.setEntityResolver(library);
		documentBuilder.setErrorHandler(new org.xml.sax.ErrorHandler() {
			public void fatalError(SAXParseException exception)
			throws SAXException {}

			public void error(SAXParseException e) throws SAXParseException {
				throw e;
			}

			public void warning(SAXParseException err) throws SAXParseException {
				System.out.println("** Warning" + ", line "
						+ err.getLineNumber() + ", uri " + err.getSystemId());
				System.out.println("   " + err.getMessage());
			}
		});
		return documentBuilder;
	}

	/** Sets the DocumentSource for Document retrieval to the given object. */
	public static void setDocumentSource(DocumentSource ds) {
		documentSource = ds;
	}

	public org.xml.sax.InputSource resolveEntity(String publicId,
			String systemId) throws SAXException, IOException {
		int n = systemId.lastIndexOf('/');
		if ((n >= 0) && (n < (systemId.length() - 1)))
			systemId = systemId.substring(n + 1);
		InputStream strm = getClass().getResourceAsStream(systemId);
		if (strm == null)
			throw new SAXException("Invalid System ID: " + systemId);
		return new InputSource(strm);
	}

	/**
	 * Returns the Hashtable whose values are loaded Documents and whose keys
	 * are the corresponding LibraryElement names.
	 */
	public static Hashtable getDocuments() {
		return documents;
	}

	/**
	 * Loads a Document from the given InputSource, strips its root node and
	 * children down to Elements only, enters it into the library and answers
	 * the new Document or <TT>null</TT> if the source cannot be parsed for
	 * some reason.
	 */
	public static org.w3c.dom.Document load(InputSource src) {
		if (src == null)
			return null;
		if (getDocumentBuilder() == null) {
			System.err.println("No DocumentBuilder available");
			return null;
		}
		try {
			Document doc = documentBuilder.parse(src);
			Element docel = doc.getDocumentElement();
			scrub(docel);
			cleanup(docel);
			String dname = docel.getAttribute("Name");
			documents.put(dname, doc);
			return doc;
		} catch (SAXParseException spe) {

			// Error generated by the parser
			System.err.println("\n** Parsing error" + ", line "
					+ spe.getLineNumber() + ", uri " + spe.getSystemId());
			System.err.println("   " + spe.getMessage());
//			Exception x = spe;
//			if (spe.getException() != null) {
//			x = spe.getException();
//			}
//			x.printStackTrace();
		} catch (SAXException sxe) {
			Exception x = sxe;
			if (sxe.getException() != null) {
				x = sxe.getException();
			}
			x.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return null;
	}

	/**
	 * New Version:
	 * TODO This method needs to be fixed 
	 * 
	 * Old version:
	 * Ensures that the given Document Element has just one VersionInfo
	 * sub-element, and that it has an FBNetwork sub-element if it is a
	 * DeviceType Document.
	 */
	private static void cleanup(Element docel) {
		/*
    	NodeList versions = docel.getElementsByTagName("VersionInfo");
    	while (versions.getLength() > 1)
    		docel.removeChild(versions.item(versions.getLength() - 1));
		 */
		if (!docel.getTagName().equals("DeviceType"))
			return;
		if(docel.getElementsByTagName("FBNetwork").getLength()>0) return;
		Element fbn = docel.getOwnerDocument().createElement("FBNetwork");
		docel.appendChild(fbn);
	}

	/**
	 * Receives string value of XML and parse it into Document 
	 * @param XML
	 */
	public static org.w3c.dom.Document parseXML(String xml){
		try {
			InputStream input = new ByteArrayInputStream(xml.getBytes());
			Document doc = documentBuilder.parse(input);
			Element docel = doc.getDocumentElement();
			scrub(docel);
			cleanup(docel);
			String dname = docel.getAttribute("Name");
			documents.put(dname, doc);
			return doc;
		} catch (SAXParseException spe) {
			// Error generated by the parser
			System.err.println("\n** Parsing error" + ", line "
					+ spe.getLineNumber() + ", uri " + spe.getSystemId());
			System.err.println("   " + spe.getMessage());
			Exception x = spe;
			if (spe.getException() != null) {
				x = spe.getException();
			}
			x.printStackTrace();
		} catch (SAXException sxe) {
			Exception x = sxe;
			if (sxe.getException() != null) {
				x = sxe.getException();
			}
			x.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return null;
	}
}
