//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.dom;

import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;

import javax.swing.JEditorPane;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.oooneida.wb.XMLCodeGenerator;
import org.oooneida.wb.dom.events.ElementSelectionEvent;
import org.oooneida.wb.translate.ParseError;
import org.oooneida.wb.translate.ParseResult;
import org.oooneida.wb.translate.STtoJava;
import org.oooneida.wb.translate.Translator;
import org.oooneida.wb.translate.TranslatorSource;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.events.Event;
import org.w3c.dom.events.EventListener;
import org.w3c.dom.events.EventTarget;
import org.w3c.dom.events.MutationEvent;

/**
 * A model for displaying a selected DOM Element in a JEditorPane using an XSLT
 * stylesheet.
 * 
 * @author JHC
 * @version 20051111/JHC - Changed ItemListener, TableListener to EventListener;
 *          extended from ElementModel.
 * @version 20050927/JHC
 */
public class DOMTranslationModel extends ElementModel implements EventListener {
    protected JEditorPane editorPane;

    protected Transformer transformer;
    protected Document document;
    protected DOMSource src = new DOMSource();
    protected TranslatorSource translatorSource;
    protected String targetLanguage;

    StreamResult dst = new StreamResult();

    public DOMTranslationModel(Document doc, JEditorPane textArea,
            String stylesheetName, String targetLanguage, TranslatorSource translatorSource) {
        super();
        this.document = doc;
        this.editorPane = textArea;
        this.translatorSource = translatorSource;
        this.targetLanguage = targetLanguage;
        
        ((org.w3c.dom.events.EventTarget) doc).addEventListener("ElementSelectionEvent", this,
                false);
        TransformerFactory tf = TransformerFactory.newInstance();
        try {
            transformer = tf.newTransformer(new StreamSource(getClass()
                    .getResourceAsStream(stylesheetName)));
        } catch (TransformerConfigurationException e) {
            messageNoTransformer();
        }
    }

    public void setElement(Element el) {
        super.setElement(el);
        ((org.w3c.dom.events.EventTarget) el).addEventListener("MutationEvent", this, false);
        if (transformer == null) {
            messageNoTransformer();
            return;
        }
        
        // Clone the dom so the original xml remains intact
        el = (Element)el.cloneNode(true);
        translate(el.getChildNodes());
        
        src.setNode(el);
        StringWriter writer = new StringWriter();
        dst.setWriter(writer);
        try {
            transformer.transform(src, dst);
            editorPane.setText(writer.toString());
        } catch (TransformerException e) {
            editorPane.setText("XSLT TransformerException occurred.");
        }
        editorPane.requestFocus();
    }

	protected void messageNoTransformer() {
        editorPane.setText("No XSLT Transformer available.");
    }

    public void handleEvent(Event evt) {
        if (evt instanceof MutationEvent)
            setElement(getElement());
        else if (evt instanceof ElementSelectionEvent)
            setElement(((ElementSelectionEvent) evt).getSelectedElement());

    }
    
    private void translate(NodeList nodes){
    	for(int i = 0; i < nodes.getLength(); i++){
    		Element node = (Element)nodes.item(i);
    		if(node.getNodeName().toLowerCase().equals("algorithm")){
    			translateAlgorithms(node);
    		}
    		else if(node.getNodeName().toLowerCase().equals("ectransition")){
    			translateGuardConditions(node);
    		}
        	translate(node.getChildNodes());
    	}
    }

    private void translateGuardConditions(Element elem) {
    	if(elem.getNodeName().toLowerCase().equals("ectransition")){
    		String stString = elem.getAttribute("Condition");
    		String guard = "";
    		if(stString.length() == 0 || stString.equals("1"))
    			return;
    		int index = stString.indexOf('&');
    		if(index == -1)
    			return;
    		guard = stString.substring(0, index);
    		stString = stString.substring(index + 1);
    		if(stString.charAt(0) == '(')
        		stString = stString.substring(1);
    		if(stString.charAt(stString.length() - 1) == ')')
        		stString = stString.substring(0, stString.length() - 1);
    		
			ParseResult result = null;
    		Translator translator = translatorSource.getTranslator("ST", targetLanguage);
			if(translator != null){
				result = translator.parse(new StringReader(stString));
			}
			else{
				return;
			}
			
			if(result != null && !result.hasErrors()){
				String newCondition = guard + "&(" + result.getSource() + ")";
				elem.setAttribute("Condition", newCondition);
			}
			else{
				String newCondition = guard + "&(" + stString + ")";
				System.err.println(elem.getNodeName() + ": " + newCondition);
			}
    	}
	}
    
    private void translateAlgorithms(Element elem){
    	if(elem.getNodeName().toLowerCase().equals("algorithm")){
    		Element codeNode = (Element)elem.getChildNodes().item(0);
    		if(codeNode.getAttribute("Text").length() > 0){
    			ParseResult result = null;
    			
    			String sourceLanguage;
    			String nodeName = codeNode.getNodeName().toLowerCase();
    			if(nodeName.equals("other")){
    				sourceLanguage = codeNode.getAttribute("Language");
    			}
    			else{
    				sourceLanguage = nodeName;
    			}
    			
    			Translator translator = translatorSource.getTranslator(sourceLanguage, targetLanguage);
    			if(translator != null){
    				result = translator.parse(new StringReader(codeNode.getAttribute("Text")));
    			}
    			else{
    				//System.out.println("Could not translate " + elem.getAttribute("Name") + ".");
    				return;
    			}
    				    		
	    		if(result != null && !result.hasErrors()){
    				elem.removeChild(codeNode);
    				
    				if(targetLanguage.equals("ST")){
    					codeNode = document.createElement("ST");
    				}
    				else if(targetLanguage.equals("LD")){
    					codeNode = document.createElement("LD");
    				}
    				else if(targetLanguage.equals("FBD")){
    					codeNode = document.createElement("FBD");
    				}
    				else{
	    				codeNode = document.createElement("Other");
	    				codeNode.setAttribute("Language", targetLanguage);
    				}
    				
    				String source = result.getSource();
    				source = XMLCodeGenerator.replaceReservedCharacters(source);
    				codeNode.setAttribute("Text", source);
    				elem.appendChild(codeNode);
    			}
    			else if(result != null){
    				System.out.println("\nAlgorithm: " + elem.getAttribute("Name") + 
    						" (" + codeNode.getNodeName() + ") parse failure.");
    				ArrayList<ParseError> errors = result.getErrors();
    				for(int i = 0; i < errors.size(); i++){
    					System.err.println(errors.get(i).toString());
    				}
    			}
    			else{
    				System.out.println("\nAlgorithm: " + elem.getAttribute("Name") + 
    						" (" + codeNode.getNodeName() + ") Language could not be translated to Java.");
    			}
    		}
    	}
    }
}
