//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.dom;

import org.w3c.dom.Element;

/**
 * A utility wrapper for Elements of the DOM tree of an IEC 61499
 * LibraryElement.
 * 
 * @author JHC
 * @version 20051017/JHC - Corrected <TT>getName(), getType(), getComment(),
 *          toString()</TT>- getAttribute() never returns null.
 * @version 20050817/JHC
 */
public class ElementModel {
    protected Element element;

    public ElementModel() {
        super();
    }

    public ElementModel(Element el) {
        super();
        element = el;
    }

    public void setElement(Element el) {
        element = el;
    }

    /**
     * Returns the Element's <TT>Name</TT> attribute. If none exists, returns
     * the Element's <TT>tagName</TT>.
     */
    public String getName() {
        String ans = element.getAttribute("Name");
        return (ans.equals("")) ? element.getTagName() : ans;
    }

    /**
     * Returns the Element's <TT>Type</TT> attribute. If none exists, returns
     * the empty String.
     */
    public String getType() {
        return element.getAttribute("Type");
    }

    public Element getElement() {
        return element;
    }

    /**
     * Returns a String representation of the contained Element. Default is a
     * <TT>Name&nbsp;:&nbsp;Type&nbsp;(*&nbsp;Comment*nbsp;*) representation if
     * possible.
     */
    public String toString() {
        String str = getName();
        String t = getType();
        if (!t.equals(""))
            str += " : " + t;
        String c = getComment();
        return (c.equals("")) ? str : str + " (* " + c + " *)";
    }

    /**
     * Returns <TT>true</TT> if <TT>obj</TT> is an ElementModel and its
     * contained Element is equal to the Element contained in this object.
     */
    public boolean equals(Object obj) {
        return (obj instanceof ElementModel)
                && (((ElementModel) obj).getElement() == element);
    }

    /**
     * Returns the Element's "Comment" attribute or other remark if anyIf none
     * exists, returns the empty String.
     */
    public String getComment() {
        return element.getAttribute("Comment");
    }

    /** Returns <TT>true</TT> if the given String is an IEC 61131-3 identifier. */
    public static final boolean isName(String s) {
        if (s == null)
            return false;
        int n = s.length();
        if (n == 0)
            return false;
        if (s.indexOf("__") >= 0)
            return false;
        if (s.charAt(s.length() - 1) == '_')
            return false;
        char c = s.charAt(0);
        boolean isLetterFirst = isLetter(c);
        if (n == 1)
            return isLetterFirst;
        if (!isLetterFirst && (c != '_'))
            return false;
        for (int i = 1; i < n; i++) {
            c = s.charAt(i);
            if ((c != '_') && !isDigit(c) && !isLetter(c))
                return false;
        }
        return true;

    }

    /**
     * Returns <TT>true</TT> if the given character is an IEC 61131-3 upper or
     * lower case letter.
     */
    public static final boolean isLetter(char c) {
        return ((c >= 'a') && (c <= 'z')) || ((c >= 'A') && (c <= 'Z'));
    }

    /**
     * Returns <TT>true</TT> if the given character is a digit in the range
     * {0..9}.
     */
    public static final boolean isDigit(char c) {
        return (c >= '0') && (c <= '9');
    }

    /**
     * Returns an <TT>ElementModel</TT> in the specified package encapsulating
     * the given DOM Element, or <TT>null</TT> if no such ElementModel exists.
     * 
     * @param el The DOM Element for which a model is requested.
     * @param packageName The period-terminated package name in which the model
     *            class (if any) is contained, e.g., <TT>
     *            "org.oooneida.wb.graph."</TT>
     */
    public static ElementModel forElement(Element el, String packageName) {
        String classname = packageName + el.getTagName();
        try {
            Class geclass = Class.forName(classname);
            Object obj = geclass.newInstance();
            if (obj instanceof ElementModel) {
                ElementModel em = (ElementModel) obj;
                em.setElement(el);
                return em;
            }
        } catch (Exception ex) {
        }
        return null;
    }
}
