//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.

//Copyright (c) 2007 University of Auckland
//Contributed to the FBench extension of the OOONEIDA Workbench project under the Common Public License

package org.oooneida.wb.graph;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLayeredPane;
import javax.swing.border.Border;

import org.oooneida.wb.dom.ElementModel;
import org.oooneida.wb.dom.events.ElementSelectionEvent;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.events.EventTarget;

/**
 * A component representing the node or edge of a directed graph corresponding
 * to an Element of the DOM of an IEC 61499 LibraryElement.
 * 
 * @author JHC, JP
 * 
 * @version 20070130/JP - Selection highlight colour has been changed from Yellow to
 * 			light blue.
 * @version 20051109/JHC - Removed ItemSelectable implementation; all
 *          notifications now via DOM Events model.
 * @version 20051012/JHC - Made <TT>getScaledAtt()</TT> result an <TT>int
 *          </TT>; moved component to front in layer when selected in <TT>
 *          setSelected()</TT>.
 * @version 20050811/JHC - Made ItemSelectable.
 * @version 20050802/JHC
 */
public abstract class GraphElement extends JComponent {
    protected ElementModel model = new ElementModel();

    private boolean selected = false;

    private ElementSelectionEvent selevt = new ElementSelectionEvent();

    protected final GraphElement me = this;
    
    public GraphElement() {
        super();
        setOpaque(true);
        setBackground(Color.white);
        setFont(GraphView.DEFAULT_FONT);
        addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent evt) {
                me.processMousePress();
                System.out.println(me.getElement().getNodeName());
            }
        });
    }

    /**
     * Returns the preferred bounds of the graph element or <TT>null</TT> if
     * the bounds cannot be computed.
     */
    public abstract Rectangle getPreferredBounds();

    /**
     * Sets the DOM Element represented by this component and updates its
     * contents.
     */
    public void setElement(Element el) {
        model.setElement(el);
        String ttt = model.toString();
        if (!ttt.equals(""))
            setToolTipText(ttt);
        removeAll();
        initComponents();
    }

    /**
     * Initializes components of this element as required. Default is to do
     * nothing.
     */
    protected void initComponents() {
    }

    /** Returns the DOM element represented by the receiver. */
    public Element getElement() {
        return model.getElement();
    }

    /**
     * Returns the default background color (<TT>Color.white</TT>) for
     * deselected elements.
     */
    protected Color getDeselectedColor() {
        return Color.white;
    }

    /**
     * Returns the default background color (<TT>Color(119, 190, 255)</TT>) for
     * selected elements.
     */
    protected Color getSelectionColor() {
        return new Color(119, 190, 255);
    }

    /**
     * Sets the selection state.
     */
    public void setSelected(boolean newState) {
        if (selected == newState)
            return;
        selected = newState;
        setBackground(selected ? getSelectionColor() : getDeselectedColor());
        if (selected) {
            Container p = getParent();
            if (p instanceof JLayeredPane)
                ((JLayeredPane) p).moveToFront(this);
        }
        repaint();
        
    }
    
    /** Returns the selection state. */
    public boolean isSelected() {
        return selected;
    }

    /**
     * Returns the center point of the given Rectangle.
     * 
     * @return The center point, or <TT>null</TT> if the given Rectangle is
     *         <TT>null</TT>
     */
    public static Point getCenter(Rectangle r) {
        if (r == null)
            return null;
        return new Point(r.x + r.width / 2, r.y + r.height / 2);
    }

    /**
     * Returns a GraphElement encapsulating the given DOM Element, or <TT>null
     * </TT> if no such GraphElement exists.
     */
    public static GraphElement forElement(Element el) {
        String classname = "org.oooneida.wb.graph." + el.getTagName();
        try {
            Class geclass = Class.forName(classname);
            Object obj = geclass.newInstance();
            if (obj instanceof GraphElement) {
                GraphElement em = (GraphElement) obj;
                em.setElement(el);
                return em;
            }
        } catch (Exception ex) {
            java.lang.System.out.println("GraphElement : "+ex);
        }
        java.lang.System.out.println("No GraphElement: " + classname);
        return null;
    }

    /** The empty border. */
    public static final Border emborder = BorderFactory.createEmptyBorder();

    public void processMousePress() {
        //if (!selected) {
            selevt.initSelectionEvent(getElement(), this,
                    ElementSelectionEvent.ACTIVATE);
            ((org.w3c.dom.events.EventTarget) getElement().getOwnerDocument())
                    .dispatchEvent(selevt);
            
        //}
    }

    /**
     * Get the named attribute, e.g. <TT>"x"</TT> from the element, scaled per
     * IEC 61499-2. If no such number exists, return 0.
     */
    public int getScaledAtt(String attname) {
        String att = getElement().getAttribute(attname);
        try {
            return (int) Math.round(Float.parseFloat(att)
                    * (getFontMetrics(getFont()).getHeight() + 4.0) / 100.0);
        } catch (NumberFormatException e) {
            return 0;
        }
    }

    /**
     * The preferred layer for plotting the receiver - one of the <TT>GraphView
     * </TT> constants EDGE_LAYER (bottom), IO_LAYER or NODE_LAYER (top).
     */
    public abstract Integer getPreferredLayer();

    /**
     * Returns <TT>true</TT> if the graph element has parameters that are
     * visible on a graph containing the element, or <TT>false</TT> (the
     * default value) if not.
     */
    public boolean hasVisibleParameters() {
        return false;
    }

    /**
     * Adds the visible parameters of this element (if any) to the given
     * GraphView.
     * 
     * @version 20070530/JP now accepts any class extending ViewModel (previously it was only for GraphView)
     */
    public void addVisibleParameters(ViewModel view) {
        if (!hasVisibleParameters())
            return;
        for (Node child = getElement().getFirstChild(); child != null; child = child
                .getNextSibling()) {
            Element el = (Element) child;
            if (el.getTagName().equals("Parameter")) {
                Parameter par = new Parameter(el, this);
                view.add(par, ViewModel.IO_LAYER);
            }
        }
    }
    
    /*
    public void addVisibleParameters(GraphView gv) {
        if (!hasVisibleParameters())
            return;
        for (Node child = getElement().getFirstChild(); child != null; child = child
                .getNextSibling()) {
            Element el = (Element) child;
            if (el.getTagName().equals("Parameter")) {
                Parameter par = new Parameter(el, this);
                gv.add(par, GraphView.IO_LAYER);
            }
        }
    }
    public void addVisibleDebugParameters(DebugView gv) {
        if (!hasVisibleParameters())
            return;
        for (Node child = getElement().getFirstChild(); child != null; child = child
                .getNextSibling()) {
            Element el = (Element) child;
            if (el.getTagName().equals("Parameter")) {
                Parameter par = new Parameter(el, this);
                gv.add(par, GraphView.IO_LAYER);
            }
        }
    }

    public void addVisibleDebugParameters(SystemView gv) {
        if (!hasVisibleParameters())
            return;
        for (Node child = getElement().getFirstChild(); child != null; child = child
                .getNextSibling()) {
            Element el = (Element) child;
            if (el.getTagName().equals("Parameter")) {
                Parameter par = new Parameter(el, this);
                gv.add(par, GraphView.IO_LAYER);
            }
        }
    }
    */
    
    /**
     * Returns the contained component with the given name or <TT>null</TT> if
     * no component with the given name is found.
     */
    public Component getComponentNamed(String compname) {
        return GraphElement.getComponentNamed(compname, this);
    }

    /**
     * Returns the component with the given name in the given Container or <TT>
     * null</TT> if no component with the given name is found.
     */
    public static Component getComponentNamed(String compname, Container cont) {
        if (cont == null)
            return null;
        for (int i = 0; i < cont.getComponentCount(); i++) {
            Component comp = cont.getComponent(i);
            if (compname.equals(comp.getName()))
                return comp;
        }
        return null;
    }
    
    protected boolean isInFBench(){
    	Container component = this;
    	boolean isInFBench = false;
    	try{
    		while(!component.getClass().toString().equals("class org.oooneida.wb.FBench")){
    			component = component.getParent();
    		}
    		isInFBench = true;
    	}
    	catch(NullPointerException e) {
    		//e.printStackTrace();
    		isInFBench = false;
    	}
    	return isInFBench;
    }
    
    protected void resetMouseListner(){
    	MouseListener[] mouseListeners = getMouseListeners();
    	for(MouseListener mouseListner : mouseListeners){
    		removeMouseListener(mouseListner);
    	}
    	processMousePress();
    }
}
