//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.tree;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JTree;
import javax.swing.SwingConstants;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeNode;

import org.oooneida.wb.IconFactory;

/**
 * A TreeNode encapsulating a file or directory of the navigation tree of a
 * library of IEC 61499 LibraryElements.
 * 
 * @author JHC
 * @version 20050901/JHC
 */
public class LibraryTreeNode extends Object implements TreeNode,
        FilenameFilter, TreeCellRenderer {
    /** The encapsulated file or directory. */
    protected File file;

    /** The component for rendering tree nodes. */
    private static JButton renderer;

    /** The parent LibraryTreeNode. */
    protected LibraryTreeNode parent;

    /** The list of allowed file suffixes. */
    protected static final String suffixes = " dtp fbt adp dev res sys xml ";

    /** The list of icon names corresponding to the file suffixes. */
    protected static final String[] iconNames = { "var", "fb16", "fb16",
            "device", "resource", "system", "xmlObject" };

    /* Returns the count of displayable children. */
    public int getChildCount() {
        return file.isDirectory() ? file.list(this).length : 0;
    }

    /**
     * Returns the string to be used to display the tree node.
     */
    public String getText() {
        String txt = file.getName();
        //int n = txt.lastIndexOf('.');
        //if ((n > 0) && (n < (txt.length() - 1)))
        //   txt = txt.substring(0, n);
        return txt;
    }

    /**
     * Returns the icon to be used for display of this element.
     */
    public Icon getIcon() {
        if (file.isDirectory())
            return IconFactory.getIcon("Open16");
        String name = file.getName();
        int n = name.lastIndexOf('.');
        if ((n < 1) || (n > (name.length() - 2)))
            return IconFactory.getIcon("default");
        String suffix = name.substring(n + 1);
        int i = 0;
        for (StringTokenizer t = new StringTokenizer(suffixes); t
                .hasMoreTokens();) {
            if (t.nextToken().equals(suffix))
                return IconFactory.getIcon(iconNames[i]);
            i++;
        }
        return IconFactory.getIcon("default");
    }

    public TreeNode getChildAt(int childIndex) {
        if (childIndex < 0)
            return null;
        File[] files = file.listFiles(this);
        if (childIndex >= files.length)
            return null;
        return new LibraryTreeNode(files[childIndex], this);
    }

    public TreeNode getParent() {
        return parent;
    }

    public int getIndex(TreeNode node) {
        if (!(node instanceof LibraryTreeNode) || !getAllowsChildren())
            return -1;
        String name = ((LibraryTreeNode) node).getFile().getName();
        String[] names = file.list(this);
        for (int i = 0; i < names.length; i++) {
            if (names[i].equals(name))
                return i;
        }
        return -1;
    }

    /*
     * Returns <TT> true </TT> if the receiver allows children.
     */
    public boolean getAllowsChildren() {
        return file.isDirectory();
    }

    public boolean isLeaf() {
        return getChildren().isEmpty();
    }

    public Enumeration children() {
        return getChildren().elements();
    }

    /**
     * Returns a LibraryTreeNode encapsulating the given file or directory with
     * the given parent node.
     */
    public LibraryTreeNode(File fl, LibraryTreeNode parent) {
        super();
        file = fl;
        this.parent = parent;
        if (renderer == null) {
            renderer = new JButton();
            renderer.setFont(new Font("SansSerif", Font.PLAIN, 10));
            renderer.setVerticalTextPosition(SwingConstants.CENTER);
            renderer.setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));
        }
    }

    /**
     * Returns a Vector of the receiver's children.
     */
    public Vector getChildren() {
        Vector ans = new Vector();
        if (!getAllowsChildren())
            return ans;
        File[] files = file.listFiles(this);
        for (int i = 0; i < files.length; i++)
            ans.addElement(getChildAt(i));
        return ans;
    }

    public boolean accept(File dir, String name) {
        if (file.isDirectory())
            return true;
        int n = name.lastIndexOf('.');
        if (n > (name.length() - 2))
            return false;
        return suffixes.indexOf(" " + name.substring(n + 1) + " ") > 0;
    }

    /** Returns the File encapsulated by this node. */
    public File getFile() {
        return file;
    }

    public Component getTreeCellRendererComponent(JTree tree, Object value,
            boolean selected, boolean expanded, boolean leaf, int row,
            boolean hasFocus) {
        LibraryTreeNode v = (LibraryTreeNode) value;
        renderer.setText(v.getText());
        renderer.setIcon(v.getIcon());
        renderer.setForeground(Color.black);
        renderer.setBackground(selected ? new Color(119, 190, 255) : Color.white);
        return renderer;
    }
}